!=========================================================================
! Copyright (C) GemTalk Systems 1986-2013.  All Rights Reserved.
!
! $Id: MessageNotUnderstood.gs 31898 2013-11-18 22:48:01Z stever $
!
!=========================================================================

removeallmethods MessageNotUnderstood
removeallclassmethods MessageNotUnderstood

category: 'For Documentation Installation only'
classmethod: MessageNotUnderstood
installDocumentation

| doc txt |
doc := GsClassDocumentation newForClass: self.

txt := (GsDocText new) details:
'The class protocol describe the behavior of the value of the global 
named MessageNotUnderstood.

This object is used to as an exception selector to catch failed message 
sends. Message not understood exceptions are resumable so any message 
in this protocol that signal such an exception may ultimately return 
to their sender.

This object is not specifed as an <exceptionSignaler> or an 
<exceptionInstantiator>. It as assumed that message not understood 
exceptions are signaled by the implemention dependent implementaton 
of the message <Object> #doesNotUnderstand:.

The instance protocol describes the behavior of exceptions that are 
signalled if the receiver of a message does not have a method with a 
matching selector.'.
doc documentClassWith: txt.

self description: doc.
%

category: 'ANSI support'
classmethod: MessageNotUnderstood
errorCategoryToCatch

	^GemStoneError.
%

category: 'ANSI support'
classmethod: MessageNotUnderstood
errorNumberToCatch

	^self errorNumberToSignal.
%

category: 'ANSI support'
classmethod: MessageNotUnderstood
errorNumberToSignal
	"RT_ERR_DOES_NOT_UNDERSTAND	No method was found for the given selector.
	  Args: (1) the  receiver; (2) the selector; (3) an array of arguments"

	^2010.
%

category: 'ANSI support'
classmethod: MessageNotUnderstood
handlesCategory: anObject number: anInteger

	^anObject == GemStoneError and: [anInteger == 2010].
%

category: 'ANSI - exceptionSignaler'
classmethod: MessageNotUnderstood
signal: anObject

	nil fooBar.	"We hope this message is not understood!"
%

category: 'ANSI - accessing'
method: MessageNotUnderstood
defaultAction
	"The current computation is terminated. The cause of the error should be
	logged or reported to the user. If the program is operating in an interactive
	debugging environment the computation should be suspended and the 
	debugger activated."

	^super defaultAction.
%

category: 'ANSI - exceptionDescription'
method: MessageNotUnderstood
isResumable
	"Message not understood errors are considered resumable, as 
	there are a number of circumstances in which we might want 
	to perform #doesNotUnderstand: handling and continue."

	^true.
%

category: 'ANSI - exceptionDescription'
method: MessageNotUnderstood
message
	"Answer the selector and arguments of the message that failed."

	^self tag
%

category: 'ANSI - exceptionDescription'
method: MessageNotUnderstood
message: aFailedMessage

	self tag: aFailedMessage.
%

category: 'ANSI - exceptionDescription'
method: MessageNotUnderstood
receiver
	"Answer the object that was the receiver of the message that failed."

	^receiver
%

category: 'ANSI - accessing'
method: MessageNotUnderstood
receiver: anObject
	"Set the object which did not understand the message to be anObject"
	
	receiver := anObject
%

category: 'ANSI - accessing'
method: MessageNotUnderstood
selector
	"Answer the selector that was not understood from the receiver's
	<failedMessage>."

	^self message selector
%

category: 'ANSI - exceptionSignaler'
method: MessageNotUnderstood
signal

	nil fooBar.	"We hope this message is not understood!"
%
